/* eslint-disable import/no-extraneous-dependencies */
const glob = require("glob")
const fs = require("fs")
const { ESLint } = require("eslint")
const ttp = require("typescript-to-proptypes")
const path = require("path")

// https://rollupjs.org/guide/en/#plugins-overview
// https://github.com/merceyz/typescript-to-proptypes/blob/master/test/index.test.ts
// https://github.com/merceyz/typescript-to-proptypes/blob/66f228715a14dd5883d7c06a2b19e6439938973f/test/index.test.ts
// https://github.com/mui-org/material-ui/blob/2231349c302a089cc556614ff562b02a729b1e77/scripts/generateProptypes.ts
// https://www.npmjs.com/package/@rollup/pluginutils
// eslint-disable-next-line jsdoc/require-returns
/**
 * @returns {{}} rollup plugin
 */
export default function propTypesFromTS() {
  return {
    name: "create-proptypes-from-ts-definition", // this name will show up in warnings and errors
    writeBundle() {
      ;(async () => {
        const tsOptions = ttp.loadConfig(
          path.resolve(__dirname, "./tsconfig.json")
        )

        const definitionFiles = glob.sync("./src/**/*.d.ts", {
          absolute: true,
          cwd: __dirname,
        })

        // Create program for all files to speed up tests
        const program = ttp.createProgram(definitionFiles, tsOptions)

        definitionFiles.forEach(async (definitionFile) => {
          if (
            !definitionFile.includes("index") &&
            !definitionFile.includes("types")
          ) {
            // const proptypes = ttp.parseFromProgram(definitionFile, program, {
            //   // shouldResolveObject: ({ name }) => {
            //   //   if (name.toLowerCase().endsWith('classes') || name === 'theme' || name.endsWith('Props')) {
            //   //     return false;
            //   //   }
            //   //   return undefined;
            //   // },
            //   checkDeclarations: true,
            // })

            // if (proptypes.body.length === 0) {
            //   return "success"
            // }

            // proptypes.body.forEach((component) => {
            //   component.types.forEach((prop) => {
            //     if (prop.name === "classes" && prop.jsDoc) {
            //       prop.jsDoc += "\nSee [CSS API](#css) below for more details."
            //     } else if (
            //       !prop.jsDoc
            //       // ||
            //       // (ignoreExternalDocumentation[component.name] &&
            //       //   ignoreExternalDocumentation[component.name].includes(prop.name))
            //     ) {
            //       prop.jsDoc = "@ignore"
            //     }
            //   })
            // })

            // let result
            // try {
            //   result = ttp.generate(ast, {})
            // } catch (error) {
            //   console.error(error)
            //   console.error("^FAILED TO GENERATE PROPTYPES FROM TS^")
            //   console.error(`error in file: ${definitionFile}`)
            //   return
            // }

            // SRC FILES: inject propTypes import
            const outputPath = definitionFile.replace(".d.ts", ".jsx")

            // https://github.com/merceyz/typescript-to-proptypes/blob/2f90fe845bd8f1e500aa16f36d5bf5184111b38d/src/parser.ts#L27-L32
            const proptypes = ttp.parseFromProgram(definitionFile, program, {
              checkDeclarations: true,
            })

            const jsContent = fs.readFileSync(outputPath, "utf8")
            const result2 = ttp.inject(proptypes, jsContent, {
              removeExistingPropTypes: true,
              babelOptions: {
                filename: outputPath,
              },
              comment: [
                "----------------------------- Warning --------------------------------",
                "| These PropTypes are generated from the TypeScript type definitions |",
                '|     To update them edit the d.ts file and run "yarn proptypes"     |',
                "----------------------------------------------------------------------",
              ].join("\n"),
              // getSortLiteralUnions,
              reconcilePropTypes: (prop, previous, generated) => {
                const usedCustomValidator =
                  previous !== undefined && !previous.startsWith("PropTypes")
                const ignoreGenerated =
                  previous !== undefined &&
                  previous.startsWith(
                    "PropTypes /* @typescript-to-proptypes-ignore */"
                  )

                if (
                  ignoreGenerated &&
                  // `ignoreGenerated` implies that `previous !== undefined`
                  previous
                    .replace(
                      "PropTypes /* @typescript-to-proptypes-ignore */",
                      "PropTypes"
                    )
                    .replace(/\s/g, "") === generated.replace(/\s/g, "")
                ) {
                  throw new Error(
                    `Unused \`@typescript-to-proptypes-ignore\` directive for prop '${prop.name}'.`
                  )
                }

                if (usedCustomValidator || ignoreGenerated) {
                  // `usedCustomValidator` and `ignoreGenerated` narrow `previous` to `string`
                  return previous
                }

                return generated
              },
              shouldInclude: ({ component, prop }) => {
                if (prop.name === "children") {
                  return true
                }
                let shouldDocument

                prop.filenames.forEach((filename) => {
                  const isExternal = filename !== definitionFile
                  if (!isExternal) {
                    shouldDocument = true
                  }
                })

                // const { name: componentName } = component
                // if (
                //   useExternalDocumentation[componentName] &&
                //   useExternalDocumentation[componentName].includes(prop.name)
                // ) {
                //   shouldDocument = true
                // }

                return shouldDocument
              },
            })
            // console.log("result2", result2)

            // 1. Create an instance with the `fix` option.
            // const eslint = new ESLint({
            //   fix: true,
            //   ignore: false,
            //   overrideConfig: { rules: { "no-undef": "off" } },
            // })

            // 2. Lint and modify text.
            // const lintResults = await eslint.lintText(result)

            fs.writeFileSync(`${outputPath}.test.jsx`, result2)
          }
        })
      })().catch((error) => {
        process.exitCode = 1
        console.error(error)
      })
    },
  }
}
